/*
 * Entry identifier (ENTRYID) functions
 *
 * Copyright (c) 2009-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBFMAPI_ENTRY_IDENTIFIER_H )
#define _LIBFMAPI_ENTRY_IDENTIFIER_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libfmapi_guid.h"

#if defined( __cplusplus )
extern "C" {
#endif

enum LIBFMAPI_ENTRY_IDENTIFIER_FLAGS
{
	LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_NOTRESERVED		= 0x08,
	LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_NOW			= 0x10,
	LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_THISSESSION		= 0x20,
	LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_NOTRECIP		= 0x40,
	LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_SHORTTERM		= 0x80
};

#define LIBFMAPI_ENTRY_IDENTIFIER_FLAG_MAPI_COMPOUND	0x80

typedef struct libfmapi_entry_identifier libfmapi_entry_identifier_t;

struct libfmapi_entry_identifier
{
	/* The flags
	 */
	uint8_t flags[ 4 ];

	/* The service provider identifier
	 */
	libfmapi_guid_t service_provider_identifier[ 16 ];

	/* The object identifier
	 */
	uint8_t *object_identifier;

	/* The object idenfier size
	 */
	size_t object_identifier_size;
};

int libfmapi_entry_identifier_from_byte_stream(
     libfmapi_entry_identifier_t *entry_identifier,
     uint8_t *byte_stream,
     size_t byte_stream_size,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

