/*
 * libpff descriptor
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_DESCRIPTOR_H )
#define _LIBPFF_DESCRIPTOR_H

#include <common.h>

#include <liberror.h>

#include "libpff_data_array.h"
#include "libpff_data_reference.h"
#include "libpff_io_handle.h"
#include "libpff_libbfio.h"
#include "libpff_tree_type.h"

#if defined( __cplusplus )
extern "C" {
#endif

int libpff_descriptor_read(
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     libpff_tree_node_t **local_descriptors,
     libpff_data_reference_t **descriptor_data_reference,
     liberror_error_t **error );

int libpff_descriptor_local_read(
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_tree_node_t *parent_local_descriptors,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     libpff_tree_node_t **local_descriptors,
     libpff_data_reference_t **descriptor_data_reference,
     liberror_error_t **error );

int libpff_descriptor_read_local_descriptors(
     uint64_t local_descriptors_identifier,
     uint8_t recovered,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     libpff_tree_node_t **local_descriptors,
     liberror_error_t **error );

int libpff_descriptor_read_data(
     uint64_t data_identifier,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     libpff_data_reference_t **descriptor_data_reference,
     liberror_error_t **error );

int libpff_descriptor_read_data_array(
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     uint8_t *array_data,
     size_t array_data_size,
     libpff_data_array_t **data_array,
     size_t *data_array_size,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

