/*
 * Index functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_INDEX_H )
#define _LIBPFF_INDEX_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libpff_libbfio.h"
#include "libpff_tree_type.h"

#if defined( __cplusplus )
extern "C" {
#endif

#define LIBPFF_INDEX_TYPE_DESCRIPTOR			0x81
#define LIBPFF_INDEX_TYPE_OFFSET			0x80

#define LIBPFF_INDEX_DESCRIPTOR_SUB_FOLDERS_CORRECTION	0x000b

typedef struct libpff_index_values libpff_index_values_t;

struct libpff_index_values
{
	/* The identifier value
	 */
	uint64_t identifier;

	union
	{
		/* Definition for offset index and descriptor index branch node
		 * and offset index leaf node
		 */
		struct
		{
			/* The file offset
			 */
			off64_t file_offset;

			/* The data size and type are used in the offset index leaf nodes
			 * The back pointer is used in the offset index and descriptor index branch nodes
			 */
			union
			{
				/* The back pointer value
				 */
				uint64_t back_pointer;

				/* Definition for offset index leaf node
				 */
				struct
				{
					/* The data size
					 */
					size32_t data_size;

					/* The reference count
					 */
					uint16_t reference_count;
				};
			};
		};

		/* Definition for a descriptor index leaf node
		 */
		struct
		{
			/* The data identifier
			 * Refers to a value in the file offset index
			 */
			uint64_t data_identifier;

			/* The local descriptors identifier
			 * Refers to a value in the file offset index
			 */
			uint64_t local_descriptors_identifier;

			/* The parent identifier
			 * Refers to a value in descriptor index
			 */
			uint32_t parent_identifier;
		};
	};
};

int libpff_index_values_free(
     intptr_t *index_values,
     liberror_error_t **error );

int libpff_index_values_compare(
     intptr_t *first,
     intptr_t *second,
     liberror_error_t **error );

int libpff_index_values_compare_allow_duplicate(
     intptr_t *first,
     intptr_t *second,
     liberror_error_t **error );

int libpff_index_node_initialize(
     libpff_tree_node_t **index_tree_node,
     liberror_error_t **error );

int libpff_index_read_node(
     libpff_tree_node_t *index_tree_node,
     libbfio_handle_t *file_io_handle,
     off64_t node_file_offset,
     int file_type,
     uint8_t *node_level,
     uint8_t *index_type,
     uint64_t *back_pointer,
     liberror_error_t **error );

int libpff_index_read_tree(
     libpff_tree_node_t *index_tree_node,
     libbfio_handle_t *file_io_handle,
     off64_t node_file_offset,
     int file_type,
     uint8_t expected_index_type,
     uint64_t expected_back_pointer,
     liberror_error_t **error );

int libpff_index_get_tree_node(
     libpff_tree_node_t *index_tree_node,
     uint64_t index_identifier,
     libpff_tree_node_t **result_index_tree_node,
     liberror_error_t **error );

int libpff_index_get_index_values(
     libpff_tree_node_t *index_tree_node,
     uint64_t index_identifier,
     libpff_index_values_t **index_values,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

