/*
 * Item tree functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_ITEM_TREE_H )
#define _LIBPFF_ITEM_TREE_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libpff_io_handle.h"
#include "libpff_list_type.h"
#include "libpff_tree_type.h"

#if defined( __cplusplus )
extern "C" {
#endif

int libpff_item_tree_node_free_recovered(
     intptr_t *item_tree_node,
     liberror_error_t **error );

int libpff_item_tree_get_tree_node_by_identifier(
     libpff_tree_node_t *item_tree_node,
     uint32_t item_identifier,
     libpff_tree_node_t **result_item_tree_node,
     liberror_error_t **error );

int libpff_item_tree_get_child_node(
     libpff_tree_node_t *item_tree_node,
     int child_node_index,
     libpff_tree_node_t **child_node,
     liberror_error_t **error );

int libpff_item_tree_get_child_node_by_identifier(
     libpff_tree_node_t *item_tree_node,
     uint32_t child_node_identifier,
     libpff_tree_node_t **child_node,
     liberror_error_t **error );

int libpff_item_tree_create(
     libpff_io_handle_t *io_handle,
     libpff_tree_node_t **item_tree_root_node,
     libpff_tree_node_t **root_folder_item_tree_node,
     libpff_list_t *orphan_node_list,
     liberror_error_t **error );

int libpff_item_tree_create_node(
     libpff_tree_node_t *descriptor_index_root_node,
     libpff_tree_node_t *descriptor_index_tree_node,
     libpff_tree_node_t *item_tree_root_node,
     libpff_list_t *orphan_node_list,
     libpff_tree_node_t **root_folder_item_tree_node,
     liberror_error_t **error );

int libpff_item_tree_get_identifier(
     libpff_tree_node_t *item_tree_node,
     uint32_t *identifier,
     liberror_error_t **error );

int libpff_item_tree_append_identifier(
     libpff_tree_node_t *item_tree_node,
     uint32_t identifier,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

