/*
 * Item values functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_ITEM_VALUES_H )
#define _LIBPFF_ITEM_VALUES_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libpff_data_reference.h"
#include "libpff_io_handle.h"
#include "libpff_libbfio.h"
#include "libpff_list_type.h"
#include "libpff_multi_value.h"
#include "libpff_table.h"
#include "libpff_tree_type.h"
#include "libpff_types.h"

#if defined( __cplusplus )
extern "C" {
#endif

typedef struct libpff_item_values libpff_item_values_t;

struct libpff_item_values
{
	/* The local descriptors
	 */
	libpff_tree_node_t *local_descriptors;

	/* The item table
	 */
	libpff_table_t *table;
};

int libpff_item_values_initialize(
     libpff_item_values_t **item_values,
     liberror_error_t **error );

int libpff_item_values_free(
     libpff_item_values_t **item_values,
     liberror_error_t **error );

int libpff_item_values_clone_copy(
     libpff_item_values_t *destination,
     libpff_item_values_t *source,
     liberror_error_t **error );

int libpff_item_values_read(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int debug_item_type,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_local_read(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_tree_node_t *parent_local_descriptors,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int debug_item_type,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_sub_node_by_identifier(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     uint32_t sub_node_identifier,
     libpff_tree_node_t **sub_node,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_number_of_sets(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     uint32_t *number_of_sets,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_number_of_entries(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     uint32_t *number_of_entries,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_entry_type(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int table_set_index,
     int table_entry_index,
     uint32_t *entry_type,
     uint32_t *value_type,
     libpff_name_to_id_map_entry_t **name_to_id_map_entry,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_entry_value(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int table_set_index,
     uint32_t entry_type,
     uint32_t *value_type,
     libpff_data_reference_t **value_data_reference,
     uint8_t flags,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_entry_value_by_utf8_name(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int table_set_index,
     uint8_t *utf8_entry_name,
     size_t utf8_entry_name_length,
     uint32_t *value_type,
     libpff_data_reference_t **value_data_reference,
     int debug_ascii_codepage,
     liberror_error_t **error );

int libpff_item_values_get_entry_value_by_utf16_name(
     libpff_item_values_t *item_values,
     uint32_t descriptor_identifier,
     uint8_t recovered,
     libpff_list_t *name_to_id_map_list,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     int table_set_index,
     uint16_t *utf16_entry_name,
     size_t utf16_entry_name_length,
     uint32_t *value_type,
     libpff_data_reference_t **value_data_reference,
     int debug_ascii_codepage,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

