/*
 * Local descriptors functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_LOCAL_DESCRIPTORS_H )
#define _LIBPFF_LOCAL_DESCRIPTORS_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libpff_io_handle.h"
#include "libpff_libbfio.h"
#include "libpff_tree_type.h"

#if defined( __cplusplus )
extern "C" {
#endif

typedef struct libpff_local_descriptor_values libpff_local_descriptor_values_t;

struct libpff_local_descriptor_values
{
	/* The local descriptor level
	 */
	uint8_t level;

	/* The identifier value
	 */
	uint64_t identifier;

	/* The data identifier value
	 * Refers to a value in the file offset index
	 */
	uint64_t data_identifier;

	/* The sub node identifier value
	 */
	uint64_t sub_node_identifier;
};

int libpff_local_descriptor_values_free(
     intptr_t *local_descriptor_values,
     liberror_error_t **error );

int libpff_local_descriptor_values_clone(
     intptr_t **destination,
     intptr_t *source,
     liberror_error_t **error );

int libpff_local_descriptors_node_initialize(
     libpff_tree_node_t **local_descriptors_tree_node,
     liberror_error_t **error );

int libpff_local_descriptors_read_node(
     libpff_tree_node_t *local_descriptors_tree_node,
     libbfio_handle_t *file_io_handle,
     off64_t node_file_offset,
     size32_t node_data_size,
     int file_type,
     uint8_t *node_level,
     liberror_error_t **error );

int libpff_local_descriptors_read_tree(
     libpff_tree_node_t *local_descriptors_tree_node,
     uint64_t local_descriptors_node_identifier,
     uint8_t recovered,
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     liberror_error_t **error );

int libpff_local_descriptors_get_tree_node_by_identifier(
     libpff_tree_node_t *local_descriptors_tree_node,
     uint64_t local_descriptor_identifier,
     libpff_tree_node_t **result_local_descriptors_tree_node,
     liberror_error_t **error );

int libpff_local_descriptors_get_values(
     libpff_tree_node_t *local_descriptors_tree_node,
     uint64_t local_descriptor_identifier,
     libpff_local_descriptor_values_t **local_descriptor_values,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

