/*
 * Message functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_MESSAGE_H )
#define _LIBPFF_MESSAGE_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libpff_extern.h"
#include "libpff_item.h"

#if defined( __cplusplus )
extern "C" {
#endif

int libpff_message_determine_attachments(
     libpff_internal_item_t *internal_item,
     liberror_error_t **error );

int libpff_message_determine_recipients(
     libpff_internal_item_t *internal_item,
     liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_entry_value_utf8_string_size(
                   libpff_item_t *item,
                   uint32_t entry_type,
                   size_t *utf8_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_entry_value_utf8_string(
                   libpff_item_t *item,
                   uint32_t entry_type,
                   uint8_t *utf8_string,
                   size_t utf8_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_number_of_attachments(
                   libpff_item_t *message,
                   int *number_of_attachments,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_attachment(
                   libpff_item_t *message,
                   int attachment_index,
                   libpff_item_t **attachment,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_attachments(
                   libpff_item_t *message,
                   libpff_item_t **attachments,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_recipients(
                   libpff_item_t *message,
                   libpff_item_t **recipients,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_plain_text_body_size(
                   libpff_item_t *message,
                   size_t *size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_plain_text_body(
                   libpff_item_t *message,
                   uint8_t *message_body,
                   size_t size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_rtf_body_size(
                   libpff_item_t *message,
                   size_t *size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_rtf_body(
                   libpff_item_t *message,
                   uint8_t *message_body,
                   size_t size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_html_body_size(
                   libpff_item_t *message,
                   size_t *size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_message_get_html_body(
                   libpff_item_t *message,
                   uint8_t *message_body,
                   size_t size,
                   liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

