/*
 * Name to ID map functions
 *
 * Copyright (c) 2010, Joachim Metz <jbmetz@users.sourceforge.net>
 * Copyright (c) 2008-2010, Joachim Metz <forensics@hoffmannbv.nl>,
 * Hoffmann Investigations.
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBPFF_NAME_TO_ID_MAP_H )
#define _LIBPFF_NAME_TO_ID_MAP_H

#include <common.h>
#include <types.h>

#include <libcstring.h>
#include <liberror.h>

#include "libpff_extern.h"
#include "libpff_io_handle.h"
#include "libpff_libbfio.h"
#include "libpff_list_type.h"
#include "libpff_types.h"

#if defined( __cplusplus )
extern "C" {
#endif

#define LIBPFF_ENTRY_TYPE_NAME_TO_ID_MAP_CLASS_IDENTIFIERS	0x0002
#define LIBPFF_ENTRY_TYPE_NAME_TO_ID_MAP_ENTRIES		0x0003
#define LIBPFF_ENTRY_TYPE_NAME_TO_ID_MAP_STRINGS		0x0004

typedef struct libpff_internal_name_to_id_map_entry libpff_internal_name_to_id_map_entry_t;

struct libpff_internal_name_to_id_map_entry
{
	/* The identifier
	 */
	uint32_t identifier;

	/* The entry type
	 */
	uint8_t type;

	union
	{
		/* The numeric value
		 */
		uint32_t numeric_value;

		/* The string value
		 */
		uint8_t *string_value;
	};

	/* The value size
	 */
	size_t value_size;

	/* Value to indicate if the string value is an ASCII string
	 */
	uint8_t is_ascii_string;

	/* The GUID containing the class identifier
	 */
	uint8_t guid[ 16 ];

#if defined( HAVE_DEBUG_OUTPUT )
	/* The debug string value
	 */
	libcstring_system_character_t *debug_string;

	/* The debug string size
	 */
	size_t debug_string_size;

#endif
};

int libpff_name_to_id_map_entry_initialize(
     libpff_internal_name_to_id_map_entry_t **name_to_id_map_entry,
     liberror_error_t **error );

int libpff_name_to_id_map_entry_free(
     intptr_t *item_descriptor,
     liberror_error_t **error );

int libpff_name_to_id_map_read(
     libpff_io_handle_t *io_handle,
     libbfio_handle_t *file_io_handle,
     libpff_list_t *name_to_id_map_list,
     liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_type(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   uint8_t *entry_type,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_number(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   uint32_t *number,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_utf8_string_size(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   size_t *utf8_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_utf8_string(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   uint8_t *utf8_string,
                   size_t utf8_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_utf16_string_size(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   size_t *utf16_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_utf16_string(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   uint16_t *utf16_string,
                   size_t utf16_string_size,
                   liberror_error_t **error );

LIBPFF_EXTERN int libpff_name_to_id_map_entry_get_guid(
                   libpff_name_to_id_map_entry_t *name_to_id_map_entry,
                   uint8_t *guid,
                   size_t size,
                   liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

